<?php
/*************************************************************************************
	mpdcmnses.lib.php : mpdweb Common Session id Library
**************************************************************************************
	ArgSessionStart() : Session Start
	ArgSessionConf()  : Make mpdweb Common Session Data
	各実行単位でのFormを統一し共有が必要な場合に使用し易くする目的となる。
	実行単位で個別な情報セットをするために各モジュールに委ねる部分がある。
	 $SESSION_CONF["sesion_id"]    : 同じIDを実行単位が共有。Libraryはmpd接続単位。
	 $SESSION_CONF["href_waitmsec"]: SESSIONをPostで設定後 location.hrefでJumpするときのWait msec 
	 $SESSION_CONF["maxlifetime"]  : セッションの有効期限 Wait sec (1440以下で使用すること)
	 mpdconfig.xml:<php_session>の設定情報。各実行単位で同様の名称を使用する。
*************************************************************************************/
function cmnses_version() { return($CMNSES_VERSION=basename(__FILE__,"")." 1.0 (20.04.2023)"); }
/**************************************************************************************
	ArgSessionStart($SESSION_CONF) : Session Start
	 $SESSION_CONF : i  : Session情報 ArgSessionConf()追加したもの。
	※HTML送出前に実行すること。各実行単位でのFormを統一するために使用する。

	※1.session.use_trans_sid:onリンクで移動全てGETでSessionIDを引き継ぐ
	    session.use_cookiesでオフにすることでクッキーの使用をしない
	    ・php_value session.use_cookies 0 , php_value session.use_trans_sid 1
	  2.PHPSESSIDがURLに出力される:{usrl}?PHPSESSID={}:対応
	    リンクで移動全てにGETでSessionIDを引き継ぐ設定と競合してしまう。
	    ・php_value session.use_cookies 0 , php_value session.use_trans_sid 0
	  mpdweb.list.phpなどリンクを考慮して(1.)を設定する。
*************************************************************************************/
function ArgSessionStart($SESSION_CONF) {
	ini_set('session.use_trans_sid',1);			// Cookie less
	ini_set('session.use_only_cookies',0);
	ini_set('session.use_cookies',0);
	ini_set('session.use_trans_sid',1);			// リンクで移動するところ全てにGETでセッションIDを引き継ぐ
	ini_set('session.gc_maxlifetime',$SESSION_CONF["maxlifetime"]); // 秒(デフォルト:1440)
	session_id($SESSION_CONF["sesion_id"]);		// SESSION IDを設定 (session_startで別IDを発行するため固定する)
	session_start();
	}

/*******************************************************************************
	ArgSessionConf($SESSION_CONF) : Make mpdweb Common Session Data
	 $SESSION_CONF: i  : XML Config一部セット済み XMLSetting 例) : array("sesion_id"=>"mpdweb","href_waitmsec"=>"50","maxlifetime"=>"1440");
	                     実行単位によって設定を変えることができるため呼出しもとで設定
	                     実行単位で，共有もあるので，形式を統一する。
	 @return      : o  : $SESSION_CONFに追加 ["sesion_id"]
	※ $XML_CONFを使用する(get_configの使用に従う)。異なるConfigを使用することで共有範囲を決定できる。
	   mpdconfig.xml:"Mpd_net":mpdの接続単位で共有する。各実行単位でmpdconfig.xmlに継承ができるので同じ共有を取れる。
	   mpdconfig.xmlではsesion_id:mpdweb-%MpdHost%となっているので準じた共有となる。変更するためには一部修正が必要。
*******************************************************************************/
function ArgSessionConf($SESSION_CONF) {
	global $XML_CONF;									// 明瞭のためglobalで定義しておく。

	$mpd_net=array("host"=>"");
	get_config("Mpd_net",$mpd_net);						// php config Display Function

	if (Commn_isIPv6($_SERVER['SERVER_ADDR'])) {
		$_SERVER['SERVER_ADDR']=gethostname();
		}
	if ($mpd_net["host"]=="localhost") {
		$mpd_net["host"] = str_replace("localhost",$_SERVER["SERVER_ADDR"],$mpd_net["host"]);	// localhostに設定時,shell実行のAddressを設定
		}
	$rep_in=array("%MpdHost%","%PHPHost%","%Client%");
	$rep_out=array($mpd_net["host"],$_SERVER["SERVER_ADDR"],$_SERVER["REMOTE_ADDR"]);
	$SESSION_CONF["sesion_id"] = str_replace($rep_in,$rep_out,$SESSION_CONF["sesion_id"]);		// 設定のsesion_id,変数を置換する
	$SESSION_CONF["sesion_id"] = str_replace(".","",$SESSION_CONF["sesion_id"]);				// localhostに設定時,shell実行のAddressを設定
	return($SESSION_CONF);								// globalが必要な場合，return(Array())をglobal項目にする。
	}
/* -------------- End Of Source --------------------------------------------------------------------------- */
?>
