<?php
/*******************************************************************************
	mpdtask.sock.lib.php : mpd web Scok Lib

	 「mpdtaskserver.pl」TaskServerとのI/F Soket Library

	Function:
	 class Sock Library:Sock_Common()/SockRead()/SockWrite()/SockDisConnect()
*******************************************************************************/
function SockCmdversion() { return(basename(__FILE__,"")." 1.0 (31.03.2024)");	}
/*******************************************************************************
	SockCmdsend($accunt,$cmd,$host,$port,$endstring,$mode,$errno,$errstr) : Sock Message Send/Receive
	$cmd      : i  : Send Message
	$host     : i  : Host Name(or IP)
	$port     : i  : Connect Port
	$timeout  : i  : Socket Timeout
	$endstring: i  : 終了判定文字列 (指定文字列受信で受信終了)
	$mode     : i  : ReturnMode , 0:Array (LFで区切り配列) , 1:String
	$errno    : io : Error No     有効時セット
	$errstr   : io : Error String 有効時セット
	@return   : o  : Failure:False / Success:Receive
	※本関数が有効となるのは,Messageが<LF>で区切られていることが前提
*******************************************************************************/
function SockCmdsend($accunt,$cmd,$host,$port,$timeout,$endstring,$mode=0,&$errno,&$errstr) {
	$Sock = new Sock_Common;
	$Sock->p_connect['host'] = $host;
	$Sock->p_connect['port'] = $port;
	$Sock->timeout = $timeout;

	if ($Sock->SockConnect() === false) { 
		global $DEBUG;if ($DEBUG !== false) { print $Sock->p_error; }	// Using Debug
		return(false); }
	if ((!is_null($accunt))&&($accunt !="")) {
		$Sock->SockWrite($accunt);			// 送信条件を引数に設定
		}
	$Sock->SockWrite($cmd);
	$getdata=$Sock->SockRead($endstring);
	$Sock->SockDisConnect();
	if (!$getdata) { return(false); }
	if ($mode !==0) { return($getdata); }
	$ary=explode("\n",$getdata);		// 受信データを改行で分割 (PHP7でsplitが廃止のため修正)
	foreach($ary as $key => $val) {
		if ($val == $endstring) {		// End Markを削除
		    unset($ary[$key]);
			break;
			}
		$ary[$key]=rtrim($val);
		if (trim($ary[$key]) =="") {	// Spaceを削除
		    unset($ary[$key]);
			}
		}
	array_values($ary);
	return($ary);
	}

/*******************************************************************************
	 class Sock_Common
********************************************************************************
	$p_connect : io  : パラメータ

	Sockコマンド関数(Net Cleass)
	    p_connect['host']:{Hostname}  :  Hostname
	    p_connect['Port']:{Port}      :  Port

  Calling Sequence
	1.$Sock = new Sock_Common:  Sock Class Create
	  $Sock->timeout=3;		:  Timeout (0:No Timeout)
	2.$Sock->connect();                       Connect
	3.$buf=$Sock->SockRead()
	4.$Sock->SockWrite($send);
	5.$Sock->SockDisConnect();
*******************************************************************************/
class Sock_Common {
	// Connection & Write Errors
	const CONNECTION_NOT_OPENED   = 100;
	const CONNECTION_FAILED       = 102;
	const WRITE_FAILED            = 103;
	public $debug                  = false;
//	public $debug                  = true;
	public $filename               = "log/mpdsock.log";
	public $logsize                = 300000;

	private   $_connection        = null;

	public $p_connect = array();
	public $p_error               = '';		// Error message
	public $w_message             = '';		// Warning message (続行用)
	public $timeout               = 0;		// 0:Timeoutチェックしない

    function __construct($host ='localhost',$port = 9010)  {	//__construct インスタンス生成時に自動コール
		$this->p_connect['host'] = $host;
		$this->p_connect['port'] = $port;
		}

/*
	SockConnect() : connection status
	 @return : o  :bool
*/
	public function SockConnect() {
		if ($this->SockIsConnect()) {							// Connect済
			return true;
			}
		$connection = @fsockopen($this->p_connect['host'], $this->p_connect['port'], $errn, $errs, 4);	// Openのタイムアウトは固定
		if ($connection) {
			$this->_connection = $connection;
			return true;
			}
		$this->p_error = 'Sock_Common : Connect Error connecting: ErrNo:'.$errn.' : '.$errs.' : CONNECTION_FAILED('.self::CONNECTION_FAILED.')';		// Error Prosess
		return false;
		}

/*
	SockIsConnect() : Check Connection Status
	 @return : o  : bool
*/
	public function SockIsConnect() {
	if (!is_resource($this->_connection)) {
		return false;
		}
	return true;
	}

/*
	SockDisConnect() : Connection close
*/
	public function SockDisConnect() {
	@fclose($this->_connection);
	}

/*
	SockWrite($data) : Send data to the socket
	 $data  : i  : command : string data to be sent
	 @return: o  :  bool
*/
	function SockWrite($data) {
	if (!$this->SockIsConnect()) {							// Are we connected?
	    $this->connect();									// Try to connect
		}
	if (!fwrite($this->_connection, $data)) {				//Write the data
		throw new Exception('Write failed : WRITE_FAILED('.self::WRITE_FAILED.')'.$data);
		}
	$this->SockTcpLog("Send  :".$data."\n");
	return true;
	}

/*
	SockRead($eof,$length=1024) : Receive Data from the Socket
	 $eof      : i  : 終了判定文字列 (指定文字列受信で受信終了)
	 $length   : i  : Receive Length 何Byteを指定しても可 (LFで終了判定文字列を生成)
	 @return   : o  : Failure:False / Success:Receive
	 ※本関数が有効となるのは,Messageが<LF>で区切られていることが前提
*/
	function SockRead($eof,$length=1024) {

	if (!$this->SockIsConnect()) {			// Are we connected?
		throw new Exception('Not connected : CONNECTION_NOT_OPENED('.self::CONNECTION_NOT_OPENED.')');
		}
	$this->w_message="";
	$ret=false;
	if ($this->timeout != 0) {
		socket_set_timeout($this->_connection, $this->timeout);
		}
	$start_time = time();
	$recv="";$buf="";
	while (!feof($this->_connection)) {			// Loop through the connection, putting the data into $line
		$c = fread($this->_connection,$length);
		$recv .= $c;
		$buf  .= $c;
		if (strpos($c,"\n") !== false ) {
			if (strpos($buf,$eof) !== false ) { break; }
			$bufstr=explode("\n",$buf);			// バッファを分割 (PHP7でsplitが廃止のため修正)
			$buf="";
			if (count($bufstr) > 0) {
				$buf =$bufstr[count($bufstr)-1];
				}
			}
		if ($this->timeout != 0) {
			$stat = socket_get_status($this->_connection);	// ソケットがタイムアウトか
 			if ($stat["timed_out"]) {
				$this->SockTcpLog("Timeout\n");
				$this->w_message="Timeout";
				$ret="Timeout";
				}
			}
		$end_time = time();
		if (($this->timeout != 0) &&(($end_time - $start_time) > $this->timeout)) {		// 受信メッセージが長すぎる
			$this->SockDisConnect();							// Bufferを捨てるため SockDisConnect / Connect
			$this->connect();
			$this->SockTcpLog("Timeout (Get too long)\n");
			$this->w_message="Timeout (Get too long)";
			$ret="Timeout (Get too long)";
			break;
			}
		$this->SockTcpLog("Recive:".$recv."\n");
		}
	if ($ret){
		return false;
		}
	return $recv;
	}

/*
	SockTcpLog($log) : MessageLog
	 $logmsg : i   : log Message Used, Debug
*/
	public function SockTcpLog($logmsg) {
	if ($this->debug) {
		$fp = fopen($this->filename, "a");
		$size = filesize($this->filename);
		if ($size > $this->logsize) {
			@rename($this->filename,$this->filename.".old");
			$fp = fopen($this->filename, "w");
			}
		fwrite($fp,date("Y/m/d H:i:s")." ".$logmsg);
    	fclose($fp);
		}
	}
}
/* -------------- End Of Source --------------------------------------------------------------------------- */
?>
