<?php
/***************************************************************************************
	mpdtool.php : Tool collection

	Function
	 func=enc    : FileEncode
	 func=SesAlt : Session Set (XMLAlternative)
	 func=ver    : Version Display
	Linkage Source
	 mpdweb.lib.php : Common Lib
 ※当システムのツールはこのソースにまとめる
****************************************************************************************/
$VERSION=basename(__FILE__)." Ver:1.3 (20.04.2023)";
$require_mod=dirname(__FILE__).'/mpdweb.lib.php';			// Mpd XML Basic Library mpdweb.lib.php
require_once($require_mod);									// Mpd : Original Library mpdwen.lib.php

$ARG=GetArgument();											// Argument Purge

if ($ARG["func"]=="enc") {									// Func:Enc : File Encode
	ArgFunc_Enc($ARG);
	}

if ($ARG["func"]=="SesAlt") {								// Func:session
	$require_ses=dirname(__FILE__).'/mpdcmnses.lib.php';	// Mpd Session id Library
	if  (file_exists($require_ses)) { require_once($require_ses); }	// Mpd mpdcmnses.lib.phpの存在確認&require
	$SESSION_CONF=GetSessionConfig();						// $XML_MPDLIBを判定する(GetEnvCommon経由)
	ArgSessionStart($SESSION_CONF);				// Start Session : Used mpdcmnses.lib.php
	ArgFunc_SesAlt($ARG);
	}
if ($ARG["func"]=="ver") {						// Func:session
	print $VERSION;
	}
/* -----------------------------------------------------------------------------
	GetArgument();
	 @return: o  : 起動引数 $_POST|$_GET $_POST優先
----------------------------------------------------------------------------- */
function GetArgument() {
	$ARG=array();									// &funcは必須パラメータ
	if (isset($_POST["func"])) { $ARG=$_POST; }			// $_POSTにURLが設定されているので採用
	else if (isset($_GET["func"])) { $ARG=$_GET; }		// $_GETにURLが設定されているので採用
	else {
		 print "mpdtool.php : Calling Parameter Error";
		 $ARG["func"]="";
		 }
	return($ARG);
	}

/* -----------------------------------------------------------------------------
	GetSessionConfig() : php Session Config
	 @return: o  : $SESSION_CONF Session Array
----------------------------------------------------------------------------- */
function GetSessionConfig() {
	global $XML_CONF;

	GetEnvCommon();
	ignore_user_abort(false);							// 切断時終了
	$SESSION_CONF=array("sesion_id"=>"mpdweb","href_waitmsec"=>"50","maxlifetime"=>"1440");
	get_config("php_session",$SESSION_CONF);			// php config php_session
	$SESSION_CONF=ArgSessionConf($SESSION_CONF);		// mpdweb:Session id 作成 : Used mpdcmnses.lib.php
	return($SESSION_CONF);
	}

/* -----------------------------------------------------------------------------
	GetEnvCommon(); Load setting/mpdconfig.xml
	 @return: o  : "":Normal !="":ErrorMessage
----------------------------------------------------------------------------- */
function GetEnvCommon() {
	global $XML_CONF;

	$XML_File = "setting/mpdconfig.xml";
		$ret="";
	try {
		$XML_CONF = get_loadconfig($XML_File);	// XML File Get FileNameはLibrary標準
		} catch( Exception $e ) {
	  $ret=$e->getMessage();
	  }
	return($ret);
	}

/*******************************************************************************
	ArgFunc_Enc($ARG); Tool Function Encode File
	 $ARG    : i  : URL Argument
	mpdtool.php?func=enc&code={code}&file={FilePath(URL)}
	code:Fileのコードを指定  SJIS/EUC/UTF-8.....

    ファイルのCode形式からシステムコードにEncodeする
    システムコードはXML:setting/mpdconfig.xml <LibConf><Codeset>{Code}
*******************************************************************************/
function ArgFunc_Enc($ARG) {
	global $XML_CONF;

	$ret=GetEnvCommon();						// Get XML Setting
	if ($ret !=="") {
		print "mpdtool.php : ".$ret."<BR>";
		return;
		}

	$LIB_CONF=array("Codeset"=>"utf-8");
	get_config("LibConf",$LIB_CONF);			 // LibConf Property mpdconfig.xmlから設定

	$ret="";
	if ((!isset($ARG["file"]))||($ARG["file"]=="")) {	// URL &file={File}が未設定時 Error
		 print "mpdtool.php : Parameter unset error(file)<BR>";
		 $ret=-1;
		 }
	if ((!isset($ARG["code"]))||($ARG["code"]=="")) {	// URL &code={Encode}が未設定時 Error
		 print "mpdtool.php : Parameter unset error(code)<BR>";
		 $ret=-1;
		 }
	if ($ret==-1) { return; }

// HTMLは最小限にする。StyleはTextをそのままに表現 <pre><code>
print <<<EOM
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="Content-Type" content="text/html;charset={$LIB_CONF["Codeset"]}" >
</head>
<body><pre><code>
EOM;

	$instr = @file_get_contents($ARG["file"]);			// FileをGet
	$outstr=$instr;
	$outstr = mb_convert_encoding($instr,$LIB_CONF["Codeset"],$ARG["code"]);	// Encode
	print $outstr;

print <<<EOM
</code></pre></body>
</html>
EOM;

	}

/*******************************************************************************
	ArgFunc_Enc($ARG); Tool Function Session Alternative
	 $ARG    : i  : [func]      :SesAlt 
	 				[Session]   :$S_SESSION[{Session}]       :$S_SESSION Name 必須
					[Key]       :$S_SESSION[{Session}][{Key}]:$S_SESSION Name 2次配列名
					[{ArrayKey}]:$S_SESSION[{Session}][{Key}]=array({ArrayKey}->val....):Arrayを作成
	XMLを変更する形式
    ※$_SESSION["XMLAlter"]["Mpd_net"]=array("host"=>"{Host}","port"=>"{Port}", 'pass' => "{Pass}");
	  $_SESSION["XMLAlter"]:mpdconfig.xml<LibConf><XMLAlternative>で設定
*******************************************************************************/
function ArgFunc_SesAlt($ARG) {
	print "mpdtool Func:SesAlt Start";

	if(!isset($ARG["Session"])) {
		  print "mpdtool.php : Fucn:SesAlt Calling Parameter Error (Shortage:Session)";
		  return;
		  }
	$seskind=$ARG["Session"];
	$seskey="";$sesary=array();$callback="";
	foreach($ARG as $key=> $val) {			// XML <Link> String
		 if (($key=="func")||($key=="Session")) { continue; }
		 if ($key=="Key") { $seskey=$val;continue; }
		 if ($key=="Callback") { $callback=$val;continue; }
		 $sesary[$key]=$val;
		 }
	if (count($sesary)==0) {
		 if ($seskey=="") {
			 unset($_SESSION[$seskind]);
			 print "mpdtool Fucn:SesAlt Crear [$seskind]";
			 }
		 else {
			 unset($_SESSION[$seskind][$seskey]);
			 print "mpdtool Fucn:SesAlt Crear [$seskind][$seskey]";
			 }
		 }
	else {
		if ($seskey=="") {
			 $_SESSION[$seskind]=$sesary;
			 }
		 else {
			 $_SESSION[$seskind][$seskey]=$sesary;
			 }
		 }
	if (isset($_SESSION['playlist'])) {
		unset($_SESSION['playlist']);
		}
	print "mpdtool Fucn:SesAlt End";
	if ($callback !=="") {
echo <<<EOM
<script type="text/javascript">
	window.parent.{$callback};
</script>
EOM;
		}
	}
/* ---------------------- End of Source ------------------------------------- */
?>
