#!/usr/bin/perl
# mpdpatch.pl
# ------------------------------------------------------------------------
#   mpdpatch.pl
#    mpdpatch.pl infile={Orignalfile} diff={Diff file} outfile={Outfile}
#    Orignalfile : PatchSource
#    Diff file   : Patch File
#                  @@ -{OrignalLine},{Line数} +{NewLine},{Line数} @@
#                   :{OrignalSource}:Space
#                  -:Delete Line
#                  +:Add Line
#    outfile     : Output File
#    diff   :Switch to default standerd input  ※diff省略時,標準入力
#    outfile:Switch to default standerd output ※diff省略時,標準出力
# ------------------------------------------------------------------------

use strict;
use warnings;

my (%EDIT)=();									# auto_update:no|yes , name:no|yes , mixer_type:no|none
my %PARAM = get_param(\@ARGV,\%EDIT);			# status:on|off start:on|off update:on|off
my $ORIGFILE=$PARAM{"infile"};
my $PATCHFILE=$PARAM{"diff"};
my $OUTFILE=$PARAM{"outfile"};

my $orig_contents;
open my $fh, '<', $ORIGFILE or die "failed to open: $!";
sysread $fh, $orig_contents,-s $fh;

my $patch_contents;
if ($PATCHFILE ne "") {
	open my $fhp, '<', $PATCHFILE or die "failed to open: $!";
	sysread $fhp, $patch_contents, -s $fhp;
	}
else{
	while (my $msg = <STDIN>) {
		$patch_contents=$patch_contents.$msg;
		}
	}

my @diffs = split(m/^(?=@@)/xms,$patch_contents);	# @@の単位に分ける
my @orig_lines = split m/\n/, $orig_contents;
push @orig_lines,"\n";						# Last \n追加

for my $diff (@diffs) {						# @@の単位ごとに処理する
	my @diff_lines = split /\n/, $diff;				# @@でかこまれた行範囲情報を取得する
	my $range_line = shift @diff_lines;
	my ($before_start, $before_amount, $after_start, $after_amount)
		= $range_line =~ m{
            ^
            \@\@ \s 
            -(\d+?),(\d+?) \s 
            \+(\d+?),(\d+?) \s
            \@\@
            $
        }xms;
	$before_start--; $before_amount--; $after_start--; $after_amount--;		# 配列の添字と行数を合わせる
	my $count = $after_start;
    for my $diff_line (@diff_lines) {
		my ($op) = $diff_line =~ m/^(.)/xms;		# 変更方法を取得
		$diff_line =~ s/^.//xms;
		if ($op eq '+') {							# +なら行を挿入
#			print join "\n","+  eq[".$count."]diff[".$diff_line."] orig[".$orig_lines[$count]."]\n";
			@orig_lines = (
				$count > 0 ? @orig_lines[0..$count-1] : (),
				$diff_line,
				@orig_lines[$count..$#orig_lines],
				);
			$count += 1;
			}
		elsif ($op eq '-') {						# -なら行を削除
#			print join "\n","-  eq[".$count."]diff[".$diff_line."] orig[".$orig_lines[$count]."]\n";
			@orig_lines = (
				@orig_lines[0..$count-1],
				$count < $#orig_lines ? @orig_lines[$count+1..$#orig_lines] : (),
				);
			}
		else{						# (' ')なら何もしない
			$count += 1;
#			print join "\n","op eq[".$count."]diff[".$diff_line."] orig[".$orig_lines[$count]."]\n";
			}

		}
	}

# Output File
if ($OUTFILE ne "") {
	if (-e $OUTFILE) {
		rename $OUTFILE,$OUTFILE.'.org';
		}
	open(my $fp, "> $OUTFILE") or die("Error:$!");		# File Truncate
	print $fp join "\n",@orig_lines;
	close $fp;
	}
else{
	print join "\n", @orig_lines;
	}

# ------------------------------------------------------------------------
#   get_param : コマンドライン引数解析
#	 $argv  : i  : Command Line Argument Array
#    $edit  : io : Command Line Argument Update:Hash
#	 @return: o  : %{hash} : Parameter Status:off/on , Start:off/on Update::off/on
# ------------------------------------------------------------------------
sub get_param {
	my ($argv,$edit)=@_;
	my %edit_cons=( "infile"=>"","diff"=>"","outfile"=>"" );

	my $help=0;
	foreach my $arg (@$argv){
		my @edtstr = split(/=/,$arg);
		my $str=$edtstr[0];
		if ((exists $edit_cons{$edtstr[0]})&&(defined $edtstr[1])) {
			$edit_cons{$edtstr[0]}=$edtstr[1];
			}
		else{
			if ($help ==0) {
				print "help : infile={infile} diff={defffile outfut={outfile}\n";
				}
			$help=1;
			}
		}
	return(%edit_cons);
	}

# ------------------------- End of Source --------------------------------
