#!/usr/bin/perl
# mpdstate.pl
# ------------------------------------------------------------------------
#   mpd stat
#    mpdstate.pl status start update="{Item}:{Value}" passwd={sudo password} mpdshell="{mpd service shall}"
#      全ての項目は省略可能。全て省略時"status"のみ指定と同じ動作
#      command:status|start|update
#       update's form:"{Item}:{Value} : Item auto_update:{yes|no} / name:{yes|no} mixer_type:{no|none}
#          ※) name:audio_output yes:/proc/asound/cardからHWの一致する名称をセット
#              mpdshell=が指定されたとき,passwdは無効。必要であれば,mpdshellに含める
#    Stdout: auto_update:no|yes
#            MPD none|on|off
#     ex) mpdstate.pl status start update="auto_update:Yes;name:yes;mixer_type:none"
#    ※ auto_update:コメント行または対象行をOn/Offのみで行追加はしない。
#      mixer_type:none/no mixer_type "none"の行をコメントにするか有効にするかを行う
#                         mixer_type:の設定がなければ追記する。
#                         mixer_type:"none"以外で有効行があれば何もしない。
#      volumioはmixer_type:"none"にしないが,BitPerfectにするためにmixer_type:"none"に設定したい対応を行う。
#      Hw番号までの対応をしていないので,厳密でないので注意。
# ------------------------------------------------------------------------
$mpdconfig="/etc/mpd.conf";						# mpd Configration
$cards="/proc/asound/cards";					# Alsa Hw
$mpdshell="/etc/init.d/mpd";					# mpd start/stop shell
$VOLMIOPW="volumio";							# sudo password default
$ARGMPDSHELL="";								# mpd start/stop 変更用 Area

my (%EDIT)=();									# auto_update:no|yes , name:no|yes , mixer_type:no|none
my %PARAM = get_param(\@ARGV,\%EDIT);			# status:on|off start:on|off update:on|off
%alsa =get_cards();
@mpdconf=get_mpdconf();

%info=get_mpdinfo(\%alsa,\@mpdconf);
if ($PARAM{"Update"} eq "on") {
	put_mpdupdate(\%info,\@mpdconf);
	}
$result=put_mpdact();							# mpd 再起動

if ($result ne true) {
	 print "MPD:none;\n";
	 }
else {
	 $result=get_mpdstate();					# mpd run status
	 if ($result eq true) {
		  print "MPD:on;\n";
	 	  }
	 else {
	 	  print "MPD:off;\n";
	 	  }
	 }

# ------------------------------------------------------------------------
#   get_param : コマンドライン引数解析
#	 $argv  : i  : Command Line Argument Array
#    $edit  : io : Command Line Argument Update:Hash
#	 @return: o  : %{hash} : Parameter Status:off/on , Start:off/on Update::off/on
# ------------------------------------------------------------------------
sub get_param {
	my ($argv,$edit)=@_;

	my %edit_cons=( "auto_update"=>"","name"=>"no","mixer_type"=>"" );
	while (my ($key,$value) = each(%edit_cons)){
		$$edit{$key}=$value;
		}

	my %PARAM=( "Status"=>"off","Start"=>"off","Update"=>"off" );
	foreach my $arg (@$argv){			# 大文字小文字区別しない
		if ($arg =~ m/^status$/i) {		# Status
			$PARAM{"Status"}="on";
			}
		if ($arg =~ m/^start$/i) {		# Start
			$PARAM{"Start"}="on";
			}
		if ($arg =~ m/^passwd=/i) {		# Password
			my @edtstr = split(/=/,$arg);
			if (defined $edtstr[1]) {
				$VOLMIOPW=$edtstr[1];
				}
			}
		if ($arg =~ m/^mpdshell=/i) {	# mpd shell
			my @edtstr = split(/=/,$arg);
			if (defined $edtstr[1]) {
				$ARGMPDSHELL=$edtstr[1];
				}
			}
		if ($arg =~ m/^update=/i) {		# Start
			my @edtstr = split(/=/,$arg);
			if (defined $edtstr[1]) {
				 $PARAM{"Update"}="on";
				 get_param_edit($edtstr[1],\%$edit);
				 }
			}
		}
	return(%PARAM);
	}

# ------------------------------------------------------------------------
#   get_param_edit : コマンドライン引数 Update Edit解析
#	 $edstr : i  : Command Line Update Argument String
#	 @return: o  :
#    ※ update's form:"{Item}:{Value} : Item auto_update:{yes|no} / name:{yes|no} mixer_type:{no|none}
# ------------------------------------------------------------------------
sub get_param_edit {
	my ($edtstr,$edit)=@_;

	my @parm = split(/;/,$edtstr);
	foreach my $par (@parm){			# 大文字小文字区別しない
#		print "Param:$par\n";
		my @one = split(/:/,$par);
		if ($one[0] =~ m/^auto_update$/i) {		# auto_update
			if (defined $one[1]) {
				if ($one[1] =~ m/^yes$/i) {		# auto_update:yes
					$$edit{"auto_update"}="yes";
					}
				elsif ($one[1] =~ m/^no$/i) {		# auto_update:no
					$$edit{"auto_update"}="no";
					}
				}
			}
		elsif ($one[0] =~ m/^name$/i) {			# name
			if (defined $one[1]) {
				if ($one[1] =~ m/^yes$/i) {			# name:yes
					$$edit{"name"}="yes";
					}
				elsif ($one[1] =~ m/^no$/i) {		# name:no
					$$edit{"name"}="no";
					}
				}
			}
		elsif ($one[0] =~ m/^mixer_type$/i) {	# mixer_type
			if (defined $one[1]) {
				if ($one[1] =~ m/^none$/i) {		# mixer_type:none
					$$edit{"mixer_type"}="none";
					}
				elsif ($one[1] =~ m/^no$/i) {		# mixer_type:no
					$$edit{"mixer_type"}="no";
					}
				}
			}
		}
	return(0);
	}

# ------------------------------------------------------------------------
#   get_cards : サウンド設定を読み込む
#	 @return : o  : %{hash} : cards key{Hw} value{Name}
# ------------------------------------------------------------------------
sub get_cards {
	my (%alsa)=();
	open(my $fp, "<", $cards) or die("error :$!");
	while (my $line = readline $fp) {
		if ($line !~ /[\[\]]/) { next; } 	# [{String}]がなければ対象外

		my $str=$line;					# 仮の名前を押さえておく[{Srting}]
		$str=~ s/\[(.*?)\]/$1/;				# "[{String}]"-->{String2}
		my $tname=$1;
		$tname=~ s/\s*$//g;					# 後ろSpaceTrim
#print "[$tname]\n";

		$line=~ s/^(.*?)\[/$1/;			# HWを得る:name "{String}"-->{String}
		my $key=$1;
		$key=~ s/ //g;						# 先頭([より前の{n}[を$keyにする
#print "$key\n";

		my $str=$line;					# Nameを得る
		$str=~ s/\]:(.*?)\n/$1/;			# "{String1}]{String2}"-->{String2}
		my $str=$1;
		$str=~ s/USB-Audio//gi;
		$name="";
		my @parts = split(/ - /s,$str);
		for (my $i = 0; $i < @parts; $i++) {
			my $str=$parts[$i];
			$str=~ s/ //g;		# ':'を削除
			if ($str eq "") { next;}
			$name=$parts[$i];				# 有効となる先頭をnameにする
			last;
			}
		my @parts = split(/USB AUDIO/i, $name);
		for (my $i = 0; $i < @parts; $i++) {
			if ($str eq "") { next;}
			$name=$parts[$i];				# 有効となる先頭をnameにする
			last;
			}
		$name=~ s/ 1.0//g;					# 長いと使えないので" 1.0"を削除(多分USB Class)
		$name=~ s/^\s*//g;					# 前SpaceTrim
		$name=~ s/\s*$//g;					# 後SpaceTrim

		if ($name eq "") {				# Nameが取れないとき仮のName($tname)を使用
			if ($tname  eq "") { next;}
			$name=$tname;
			}
		$alsa{$key}= $name;
#printf "[%s]%s\n",$key,$name;
		}
	close $fp;
	return(%alsa);
	}

# ------------------------------------------------------------------------
#   get_mpdconf : mpd.confを読み込む
#	 @return : o  : @mpdconf{Table} : Line
# ------------------------------------------------------------------------
sub get_mpdconf {
	my @mpdconf;

	open(my $fp, "<", $mpdconfig) or die("error :$!");
	while (my $line = readline $fp) {
		chomp($line);		# NLを削除
		push @mpdconf,$line;
#		print "$line\n";
		}
	close $fp;
	return(@mpdconf);
	}

# ------------------------------------------------------------------------
#   get_mpdstate : mpd action check
#    @return: o  : true;mpd Act false:mpd stop
# ------------------------------------------------------------------------
sub get_mpdstate {
	my $ret=false;
	my @result = `ps -ef | grep '/usr/bin/mpd' | grep -v grep`;
	foreach my $line(@result) {
		my @lword = split(/ /, $line);
		foreach my $lary (@lword) {
			if ($lary eq '/usr/bin/mpd' ) {
				$ret=true;
				last;
				}
			}
	    }
	return($ret);
	}

# ------------------------------------------------------------------------
#   get_mpdconf : mpd.confを読み込む
#    $alsa    : i  : scalar: %{hash} : %alsa cards key{Hw} value{Name}
#    $mpdconf : i  : scalar: ${array}: @mpdconf{Table}
# ------------------------------------------------------------------------
sub get_mpdinfo {
	my ($alsa,$mpdconf)=@_; 
	my (%info)=();

	my $updidx="";my $outidx="";my $nameidx="";my $mtypecmtidx="";my $hwname="";my $hwidx="";my $mtypeidx="";my $mtypecmtidx="";$mtypeadd="";
	for (my $i = 0; $i < @$mpdconf; $i++) {
		my $str=$mpdconf[$i];
		$str =~ s/\t//g;								# 'Tab'を削除
		$str =~ s/ //g;
		if ($str eq "") { next; }						# SpaceLine
#		printf"[%2s]%s\n",$i,$mpdconf[$i];
		$str =~ s/ //g;
		if (($str !=~ /^#/)&&($str !~ /}/)) { $mtypeadd=$i; }	# Without Comment
		my $check=$str;
		$check=~ s/#*(.*?)/$1/;							# 前の#を削除(Space/Tabが削除済みなのでパラメータが先頭になる)
		if ($check =~ /^auto_update"/) {					# auto_update Search
			if ($str =~ /^#/) {
				 if ($updidx eq "") {
					 $updidx=$i;
					 }
				 }
			else {
				 $updidx=$i;
				 }
			}
		if ($str =~ /^audio_output/) {					# audio_output Search
			$hwidx="";$nameidx="";$hwname="";$mtypeidx="";
			$outidx=$i;
			}
		if (($outidx ne "")&&($str =~ /^name/)) {			# And name Search
			$nameidx=$i;
			}
		if (($outidx ne "")&&($str =~ /^device/)) {			# And device Search
			$str=~ s/hw:(.*?)"/$1/;								# Format hw:{n}:{m},
			my @list = split(/,/,$1);
			if (exists $alsa{$list[0]}) {						# hw Cooperate
				$hwname=$alsa{$list[0]};
				$hwidx=$i;
			} else {
			    $outidx="";$nameidx="";$mtypecmtidx="";$hwname="";$mtypeidx="";
				}
			}
		if (($outidx ne "")&&($str =~ /^mixer_type/)) {		# And mixer_type Search
			$mtypeidx=$i;
			}
		if (($outidx ne "")&&($str =~ /^#/)&&($str =~ /mixer_type/)&&($str =~ /"none"/)) {		# And mixer_type comment Search
			$mtypecmtidx=$i;
			}

		if ($str =~ /^#/) { ;next; }						# Without Comment
		if (($outidx ne "")&&($str =~ /}/)) {			# One Device End
			if ($hwidx ne "") {
				if ($nameidx ne "") {						# Keep:Name
					my $tmpstr=$mpdconf[$nameidx];
					$tmpstr=~ s/\"(.*?)\"/$1/;				# name "{String}"-->{String}
					my $chgstr=$mpdconf[$nameidx];
					if ($hwname ne "") {
						$chgstr =~ s/$1/$hwname/g;
						}
					$info{$nameidx}= $chgstr;
					}
			 	if ($EDIT{"mixer_type"} eq "no") {	# --- when mixer_type:no !!		# mixer_type:none以外で生きている行があればなにもしない
					 if (($mtypeidx ne "")&&($mpdconf[$mtypeidx] =~ /"none"/)) {
						$info{$mtypeidx}="#$mpdconf[$mtypeidx]";	# mixer_typeの生きている行がコメント
						}
					 }
				else {								# --- when mixer_type:none !!
					 if ($mtypeidx ne "") {						# mixer_typeの生きている行があればなにもしない
					 	}
					 elsif ($mtypecmtidx ne "") {					# Commentを生かす(mixer_type;noneのコメント行)
					    my $tmp=$mpdconf[$mtypecmtidx];
						$tmp =~ s/\t*(.*?)/$1/;						# " " , 'Tab'を削除
						$tmp =~ s/ *(.*?)/$1/;
						$tmp=~ s/#*(.*?)/$1/;						# 前の#を削除(Space/Tabが削除済みなのでパラメータが先頭になる)
						$tmp =~ s/\t*(.*?)/$1/;						# " " , 'Tab'を削除
						$tmp =~ s/ *(.*?)/$1/;
						$info{$mtypecmtidx}="\t\t$tmp";
						}
					 elsif ($mtypeadd ne "") { $info{$mtypeadd+0.5}= "\t\tmixer_type\t\"none\"\t# Add by mpdweb";
					 	}
					 }

				$outidx="";$hwidx="";$nameidx="";$mtypecmtidx="";$hwname="";$mtypeidx="";
				}
			}
		}
	if ($updidx ne "") {								# Keep Update
		 $info{$updidx}= $mpdconf[$updidx];
		 $str = $mpdconf[$updidx];
		 $str=~ s/ *(.*?)/$1/;							# 前の " " を削除(Space/Tabが削除済まのでパラメータが先頭になる)
		 $str=~ s/\t*(.*?)/$1/;							# 前の "\t"を削除(Space/Tabが削除済まのでパラメータが先頭になる)
		 my $now=$str;
		 if ($EDIT{"auto_update"} ne "") {
			 if ($str =~ /^#/) {
				  if ($EDIT{"auto_update"} eq "yes") {
					  $str=~ s/#*(.*?)/$1/;					# 前の#を削除(Space/Tabが削除済まのでパラメータが先頭になる)
					  $info{$updidx}= $str;
					  $now=$str;
					  }
				  }
			 elsif ($mpdconf[$updidx] =~ /"yes"/gi) {			# auto_update:Live Line
				  if ($EDIT{"auto_update"} eq "no") {			# auto_updateを#にする
					  $info{$updidx}= "#$mpdconf[$updidx]";
					  $now=$info{$updidx};
					  }
				  }
			 }
		 my $upsts="on";								# auto_update結果をConsole
		 if ($now =~ /^#/) {									# auto_update:コメント
			   $upsts="off";
			   }
		 elsif ($now =~ /"yes"/gi) { } else {					# auto_update:Live Line Yesでない
			   $upsts="off";
			   }
		 printf "auto_update:%s;\n",$upsts;
		 }
	else {			# "auto_updateLine"(コメント含む)がなければ更新しないので固定
		 printf "auto_update:off;\n";
		 }
	return(%info);
	}

# ------------------------------------------------------------------------
#   put_mpdupdate : mpd.conf Updatye
# ------------------------------------------------------------------------
sub put_mpdupdate {
	my ($info,$mpdconf)=@_; 
	my (%mpdupdate)=();
	my $exchange="no";
	for ($i=0;$i < @$mpdconf; $i++) {					# Use Update hash
		my $nkey=sprintf "%05.1f",$i;
		$mpdupdate{$nkey}=@$mpdconf[$i];
		}
	for my $key (sort keys %$info) {
		my $str=$info{$key};
		my $nkey=sprintf "%05.1f",$key;
		if (exists $mpdupdate{$nkey}) {					# Add Line ?
			if ($str =~ /auto_update/) {					# auto_update : Update
			 	if ($EDIT{"auto_update"} eq "") { ;next; }
				}
			if ($str =~ /name/) {							# name : Update
			 	if ($EDIT{"name"} eq "no") { next; }
				$str=~ s/\"(.*?)\"/$1/;						# name "{String}"-->{String}
				my $newname=$1;
				my $tmpstr=$mpdconf[$key];
				$tmpstr=~ s/\"(.*?)\"/$1/;
				my $oldname=$1;
				$str = $mpdconf[$key];
				$str=~ s/$oldname/$newname/g;
				}
			if ($str =~ /mixer_type/) {					# mixer_type : Update
			 	if ($EDIT{"mixer_type"} eq "") { next; }
				}
			}
		if ($mpdupdate{$nkey} ne $str) {
			$exchange="on";
			$mpdupdate{$nkey}=$str;
			}
#		printf "[%s]%s\n",$nkey,$str;
		}
	if ($exchange eq "no") { return }
	put_mpdbackup();
	my $fkeep="$mpdconfig.now";							# su でないとetc/下にファイルを作成できないのでOriginalをコピーしダイレクトに作成
	$cmdstr = "echo $VOLMIOPW | sudo -S cp $mpdconfig $fkeep";
	my $cmd = `$cmdstr`;
	$cmdstr = "echo $VOLMIOPW | sudo -S chmod 0666 $mpdconfig";
	my $cmd = `$cmdstr`;

	open($fp, "> $mpdconfig") or die("Error:$!");
	for my $key (sort keys %mpdupdate) {
		my $str=sprintf "%s\n",$mpdupdate{$key};
		print $fp "$str";
		}
	close $fp;
	}

# ------------------------------------------------------------------------
#   put_mpdbackup : mpd.conf Backup
# ------------------------------------------------------------------------
sub put_mpdbackup {
	my $fback="$mpdconfig.org";
	if (-f $fback) {
#		 print "Backup Skip\n";
		 }
	 else {
#		print "Make Backup\n";
		$cmdstr = "echo $VOLMIOPW | sudo -S cp $mpdconfig $fback";
		my $cmd = `$cmdstr`;
		}
	}

# ------------------------------------------------------------------------
#   put_mpdact : mpd service Start
#    @return: o  : true;mpd kick false:mpd shell nothing
# ------------------------------------------------------------------------
sub put_mpdact {
	my $ret=false;
	my $cmdstr="";
	if (-f $mpdshell) {
		 if ($PARAM{"Start"} eq "on") {
			if ($ARGMPDSHELL eq "") {
				 $cmdstr = "echo $VOLMIOPW | sudo -S $mpdshell restart";
				 }
			else {
				 $cmdstr = $ARGMPDSHELL;
				 }
#			print "shell[$cmdstr]\n";
			my $cmd = `$cmdstr`;
#			print $cmd;
			}
		 $ret=true;
		 }
	else {
#		 print "not '$mpdshell'\n";
		 }
	return($ret);
	}
# ---------- End of Source ------------------------------------------------

# - Debug --------------------------------------------------------------
#print "PARAM\n";
#while (my ($key,$value) = each(%PARAM)){
#	print "key=$key,value=$value\n";
#	}
#print "EDIT\n";
#while (my ($key,$value) = each(%EDIT)){
#	print "key=$key,value=$value\n";
#	}
#print "mpdconf\n";
#foreach my $ary (@mpdconf) {					# Array: mpdconf mpd.comf Dump
#	print "$ary\n";
#	}

#print "alsa\n";
#while (my ($key,$value) = each(%alsa)){		# Hash : Alsa Dump
#	print "key=$key,value=$value\n";
#	}

#print "info\n";
#while (my ($key,$value) = each(%info)){			# Hash : Update Dump
#	print "key=$key,value=$value\n";
#	}

exit(0);
